#ifndef atom_table_h__included
#define atom_table_h__included

#include <windows.h>
#include "app_opt.h"
#include "console.h"
#include "str.h"

struct atom_s;

struct list_s	// Structure LIST.
{
	atom_s *item;
	union
	{
		list_s *sublist;	// list can content lists inside.
		atom_s *value;		// this field can be just an atom value.
	};
	list_s *next;
};

enum atom_s_info				// atom_s.info possible values.
{
	atom_info_none		= 0x000000,
	atom_info_external	= 0x010000,		// instead of ATOM_S_EXTERNAL
	atom_info_function	= 0x020000,		// set if atom is function.
	//atom_info_system	= 0x080000		// system function (don't show that function in export list for library in "ATOL: Run WARNING" dialog).
	atom_info_spy		= 0x100000		// set spy point on given function.
};

struct atom_s	// structure ATOM.
{
	str_s   name;		// str_s structure.
	atom_s *next;		// pointer to the next one atom.
	// FOR FUNCTIONS:
	list_s *args;		// function arguments or ATOM_S_EXTERNAL for extern functions.
	list_s *body;		// function body or address of external function
	int info;			// set of useful flags.

	int flag(atom_s_info);			// check whether flag is set or not.
	int notflag(atom_s_info);		// not flag();
	void set_flag(int);				// set a flag.
	void clear_flag(atom_s_info);	// clear a flag.
};

struct scope_s
{
	list_s  *localvars;
	atom_s  *prefix;
	int		func; // true if it's the function's local scope.

	scope_s();
	scope_s(scope_s **prev, atom_s *pref, int in_func = 0);
	void done();
	atom_s * new_var(atom_s *name, atom_s *val);
	list_s * get_var(atom_s *name);
	list_s * find_var(atom_s *name);
	atom_s ** var_value(atom_s *name);

protected:
	scope_s **p_prev_scope, *prev_scope;
};


atom_s * atoms_new(); // create new unnamed atom.

atom_s * atoms_add(char * name);
atom_s * atoms_add(char chr);
atom_s * atoms_add(str_s *);

int atoms_count();

atom_s * atoms_find(char * name);
atom_s * atoms_find(str_s * name);

// LIST FUNCTIONS:
list_s * list_new(char *atom);
list_s * list_new(atom_s *atom);
list_s * list_add(list_s **plist, char *atom);
list_s * list_add(list_s **plist, atom_s *atom);
list_s * list_push(list_s **plist, atom_s *atom);
list_s * list_last(list_s *list);
void list_free(list_s *list);
list_s * list_find(list_s *list, atom_s *item);
list_s * list_get(list_s *list, int index);
list_s * list_null_sublists(list_s *list);

#ifdef APP_OPT_DEBUG
void atoms_view();
void list_view(int tab, list_s *p);
#endif

int list_length(list_s *list);


#endif // atom_table_h__included
